@ECHO off
SETLOCAL
REM (c)2013 Schwindt CAD/CAM-Technologie GmbH
REM
REM  Name              : update_slave_catrep.bat
REM  Version           : 1.02
REM
REM  Created           : 18.01.2012 Walter Laub (wla, walter.laub@schwindt.eu)
REM  Last Modification : 14.06.2013, wla
REM
REM  Intention         : replicates CATREP from one MASTER replica to
REM                              a SLAVE replica
REM                      !!!  This script is intended to work only with   !!!
REM                      !!!  our strucures in share CATREP               !!!
REM
REM  Require           : robocopy.exe (MS) in search path of MASTER
REM                             (copy one to %WINDIR%\system32)
REM
REM  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
REM  !!!        This script is delivered as a sample for your own!         !!!
REM  !!!         Maybe changes are necessary for your environment!         !!!            
REM  !!!            Please take care about timing replications !           !!!
REM  !!!                                                                   !!!
REM  !!! If a client runs a replication from the SLAVE_CATREP and in the   !!!
REM  !!! meantime a new replication of CATREP is started from MASTER to    !!!
REM  !!! SLAVE, the client will become inconsitent.                        !!!
REM  !!! To avoid this problem, please take care about timing replications !!!
REM  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
REM
REM  HISTORY :	
REM
REM  Syntax: update_slave_catrep.bat [MASTER_CATREP] [SLAVE_CATREP]
REM    where MASTER_CATREP and SLAVE_CATREP are CATREP replicas.
REM    LOCAL as well UNC directory notation is supported for MASTER_CATREP!
REM    Only UNC directory notation is supported for SLAVE_CATREP!
REM    Space character in path is not supported!
REM
REM  Short Description:
REM    The batch routine first check the syntax and some prerequisites.
REM    The file [SLAVE_CATREP]\update_catrep_allowed.txt indicate the 
REM    acceptance of updates. You must create it manually before 1st
REM    replication run!
REM    Then the file [SLAVE_CATREP]\slave_in_update.txt will be created.
REM    The directories cus5, fdec, pro will be robocopied to [SLAVE_CATREP].
REM    If the operations finish with no errors, at last the files update.txt
REM    and update_soft.txt will be copied to [SLAVE_CATREP] and the file
REM    [SLAVE_CATREP]\slave_in_update.txt will be deleted.
REM
REM    Please start this batch periodically with the windows scheduler.
REM    Take care about the user id that will execute it, configure scheduler
REM    adequate. It is recommended to have a special user catrep(?) for this,
REM    with no password change, which is not used in other scope!
REM    Be safe that only one instance of script is running!
REM
REM    Running CABUM-Starter from client side, it's start will check for the
REM    file [CATREP]\slave_in_update.txt with adecvate actions.
REM    Please remark, for this you may be need a new CABUM version.
REM

REM --->>> Your customization starts here <<<---
REM No customization needed
REM <<<---  Your customization ends here  >>>---

REM MAIN
SET MASTER=%1
SET SLAVE=%2
SET MyLOCK=%SLAVE%\slave_in_update.txt
SET UPDATE_ALLOWED=update_catrep_allowed.txt

ECHO.
ECHO (c)2013 Schwindt CAD/CAM-Technologie GmbH
ECHO                  Callenberger Str. 8
ECHO                  D-96450 Coburg
ECHO                  Germany
ECHO                  Phone  +49 9561 5560 0
ECHO                  mailto info@schwindt.eu
ECHO.

SET ERR=0
IF NOT .%MASTER% ==. IF NOT .%SLAVE% ==. IF NOT .%MASTER% ==.-h IF NOT .%MASTER% ==.-H IF NOT .%MASTER% ==./h IF NOT .%MASTER% ==./H IF NOT .%MASTER% ==./? GOTO :CHECK1
GOTO :HELP

REM Some Checks
:CHECK1
REM robocopy.exe
robocopy.exe /? | findstr "\/MIR" 1>NUL 2>&1
IF NOT ERRORLEVEL 1 GOTO :CHECK2
  ECHO Robocopy not found! Please check!
  GOTO :CHECKFAIL

:CHECK2
REM MASTER_CATREP
IF EXIST "%MASTER%" GOTO :CHECK2A
  ECHO Please check for MASTER_CATREP: %MASTER%!
  GOTO :CHECKFAIL
:CHECK2A
IF EXIST "%MASTER%\cus5" IF EXIST "%MASTER%\fdec" IF EXIST "%MASTER%\pro" IF EXIST "%MASTER%\update.txt" GOTO :CHECK2B
  ECHO Please check for valid structures of MASTER_CATREP: %MASTER%!
  GOTO :CHECKFAIL
:CHECK2B
IF NOT EXIST "%MASTER%\%UPDATE_ALLOWED%" GOTO :CHECK3
  ECHO This seems to be a SLAVE replica! Replication is supported only from MASTER!
  GOTO :CHECKFAIL

:CHECK3
REM SLAVE_CATREP
IF "%SLAVE:~0,2%" == "\\" GOTO :CHECK3A
  ECHO Only UNC path notation is supported for SLAVE_CATREP!
  GOTO :CHECKFAIL
:CHECK3A
IF EXIST "%SLAVE%" GOTO :CHECK4
  ECHO Please check for SLAVE_CATREP: %SLAVE%! The directory must exist!
  GOTO :CHECKFAIL
:CHECK4
REM Check if Updates are allowed
IF EXIST "%SLAVE%\%UPDATE_ALLOWED%" GOTO :LOCK_CATREP
  ECHO For security reasons the file %SLAVE%\%UPDATE_ALLOWED% must exist!
  ECHO Creating manually this file you will accept that the directory will be updated!
  GOTO :CHECKFAIL

:LOCK_CATREP
REM create the lock file
ECHO Created from %COMPUTERNAME% with batch routine %~nx0!                    > "%MyLOCK%" 2>NUL
ECHO.                                                                        >> "%MyLOCK%" 2>NUL
ECHO The presence of this file indicate that a replication from              >> "%MyLOCK%" 2>NUL
ECHo the master replica %MASTER%                                             >> "%MyLOCK%" 2>NUL
ECHO to   slave replica %SLAVE% is running right now!                        >> "%MyLOCK%" 2>NUL
ECHO.                                                                        >> "%MyLOCK%" 2>NUL
ECHO Time stamp: %DATE%, %TIME%                                              >> "%MyLOCK%" 2>NUL
ECHO Remove this file only if you know what you do!                          >> "%MyLOCK%" 2>NUL
ECHO Normally the time stamp should be less than a few hours!                >> "%MyLOCK%" 2>NUL
IF NOT ERRORLEVEL 1 GOTO :CHECK4A
  ECHO Cannot create file %MyLOCK%. Please check!
  ECHO Please verify write rights for entire directory %SLAVE%!
  GOTO :CHECKFAIL

:CHECK4A
ECHO Start: %DATE%, %TIME%                                                   >> "%SLAVE%\%UPDATE_ALLOWED%" 2>NUL
IF NOT ERRORLEVEL 1 GOTO :CHECKRDY
  ECHO Cannot write to file %SLAVE%\%UPDATE_ALLOWED%. Please check!
  ECHO Please verify write rights for entire directory %SLAVE%!
  GOTO :CHECKFAIL

  
:CHECKFAIL
ECHO.
ECHO !!! Cancel replication !!!
ECHO.
:HELP
ECHO.
ECHO Usage:
ECHO   update_slave_catrep.bat [MASTER_CATREP] [SLAVE_CATREP]
ECHO.
ECHO Example:
ECHO   update_slave_catrep.bat d:\export\catrep\cl \\SERVER\catrep\cl
ECHO.
EXIT /B 1


:CHECKRDY
SET ROBO_OPT=/MIR /NFL /NDL /R:0
robocopy.exe /? | findstr /I "\/MT" 
IF NOT ERRORLEVEL 1 SET ROBO_OPT=%ROBO_OPT% /MT

REM robocopy.exe returns an exit code ge 8 if an error occures
ECHO Start replication for %MASTER%\cus5 to %SLAVE%\cus5
robocopy.exe "%MASTER%\cus5" "%SLAVE%\cus5" %ROBO_OPT%
IF ERRORLEVEL 8 SET ERR=1

ECHO Start replication for %MASTER%\fdec to %SLAVE%\fdec
robocopy.exe "%MASTER%\fdec" "%SLAVE%\fdec" %ROBO_OPT%
IF ERRORLEVEL 8 SET ERR=1

ECHO Start replication for %MASTER%\pro to %SLAVE%\pro
robocopy.exe  "%MASTER%\pro"  "%SLAVE%\pro"  %ROBO_OPT% /XF cabum.bat
IF ERRORLEVEL 8 SET ERR=1

ECHO.
IF .%ERR% == .1 GOTO :ERR
ECHO Copy and adjust cabum.bat
FOR /F "tokens=1 delims=\" %%a in ("%SLAVE%") DO SET SLAVE_SERVER=\\%%a

SET DUMMY=%TEMP%\cabum%RANDOM%.bat
ECHO REM This file was created by %COMPUTERNAME% running batch %~nx0                    > "%DUMMY%"
ECHO REM Do not modify!                                                                >> "%DUMMY%"
ECHO REM Timestamp: %DATE%, %TIME%                                                     >> "%DUMMY%"
ECHO.                                                                                  >> "%DUMMY%"
IF EXIST "%MASTER%\pro\Cabum6\Bin\cabum.bat" FOR /F "tokens=1,* delims=]" %%a in ('"type "%MASTER%\pro\Cabum6\Bin\cabum.bat"|FIND.EXE /N /V """') DO (
  SET TXT=%%b
  SETLOCAL ENABLEDELAYEDEXPANSION
  IF /I "!TXT:~0,20!" == "SET CATREP_SERVER=\\" SET TXT=SET CATREP_SERVER=%SLAVE_SERVER%
  >> "%DUMMY%" ECHO(!TXT!
  ENDLOCAL
)
COPY /B /Y /Z "%DUMMY%" "%SLAVE%\pro\Cabum6\Bin\cabum.bat" 1>NUL 2>&1
DEL /F "%DUMMY%"

ECHO Copy update.txt and update_soft.txt from %MASTER% to %SLAVE%
XCOPY "%MASTER%\update.txt"      "%SLAVE%" /Q /H /Y /R /Z 1>NUL 2>&1
XCOPY "%MASTER%\update_soft.txt" "%SLAVE%" /Q /H /Y /R /Z 1>NUL 2>&1
DEL /F "%MyLOCK%"
GOTO :NOERR

:ERR
ECHO Some errors are detected during replication! Please check!
ECHO.
EXIT /B 1

:NOERR
ECHO Stop:  %DATE%, %TIME%                                                   >> "%SLAVE%\%UPDATE_ALLOWED%" 2>NUL
ECHO.                                                                        >> "%SLAVE%\%UPDATE_ALLOWED%" 2>NUL


:END
ECHO.
EXIT /B 0
